# --8<--8<--8<--8<--
#
# Copyright (C) 2007 Smithsonian Astrophysical Observatory
#
# This file is part of mst-cpan
#
# mst-cpan is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# mst-cpan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the 
#       Free Software Foundation, Inc. 
#       51 Franklin Street, Fifth Floor
#       Boston, MA  02110-1301, USA
#
# -->8-->8-->8-->8--

package MST::CPAN::Handler;

use strict;
use warnings;

use CPAN qw();

use SysArch;
use SysPathSubst;
use File::Spec::Functions qw( catfile );


sub dispatch
{
    my ( $mod, $module_name ) = @_;

    # default
    my $modhandler = 'MST::CPAN::Handler';

    # let's see if we provide special handling. convert '::' to '_'
    # to keep the paths manageable
    ($modhandler = $module_name) =~ s/::/_/g;
    $modhandler ='MST::CPAN::Handler::' . $modhandler;

    # try to load the handler.
    eval "require $modhandler";

    # error, use default handler
    if ( $@ )
    {
	my $err = $@;

	die $@
	  unless $err =~ /can't locate/i;

	$modhandler = 'MST::CPAN::Handler';
    }

    my $mh = $modhandler->new($mod);
    local %ENV = %{$mh->{ENV}};
    local $CPAN::Config = { %{$CPAN::Config} };
    $CPAN::Config->{makepl_arg} =
      join(' ', $CPAN::Config->{makepl_arg}, @{$mh->{makepl_arg} } );

    $mh->get;
    $mh->patch;
    $mh->install;
}

# Handler->new( $mod );
sub new
{
    my $class = shift;
    my $mod = shift;

    my $self = bless { }, $class;

    $self->{mod} = $mod;
    $self->{force_install} = 0;
    $self->{ENV} = {%ENV};
    $self->{makepl_arg} = [];

    $self->init( @_ );
    return $self;
}

sub init
{
}

sub get
{
    $_[0]->{mod}->get;
}

sub id
{
    $_[0]->{mod}->id;
}

sub mod
{
    $_[0]->{mod};
}

sub force_install
{
    my $self = shift;

    $self->{force_install} = shift
      if $@;

    return $self->{force_install};
}

sub makepl_arg
{
    my $self = shift;
    push @{$self->{makepl_arg}}, @_;

    return;
}

sub fix_file
{
    my $self = shift;
    my $file = shift;

    my $dir = $self->{mod}->dir;

    my $fh;

    warn( "Fixing $dir/$file\n" );

    open( $fh, "$dir/$file" )
      or die( "unable to open $dir/$file\n" );

    my @lines = <$fh>;
    close $fh;


    while( @_ )
    {
        if ( 'CODE' eq ref $_[0] )
        {
            my $sub = shift;
            $sub->($_) foreach @lines;
        }
        else
        {
            my( $pat, $rep ) = splice( @_, 0, 2 );
            $pat =~ s/(\W)/\\$1/g
              unless ( 'Regexp' eq ref $pat );
            s{$pat}{$rep} foreach @lines;
        }
    }


    open( $fh, ">$dir/$file" )
      or die( "unable to open $dir/$file\n" );

    print $fh @lines;
    close $fh;
}

sub fix_MakefilePL
{
    my ( $self ) = shift;

    $self->fix_file( 'Makefile.PL', @_ );
}


#----------------------------------------------------------------------#

sub install
{
    my ( $self ) = @_;

    my $mod = $self->mod;

    # this bizarrity is to work around the fact that we can't force a
    # "make install" using the standard channels with force
    # (force_update) on without the build directory getting scrubbed and
    # thus losing our previous modifications. This seems to be the only
    # path through the CPAN logic which results in what we want.

    if ( $::opt{force} )
    {

	$mod->make;

	my $dist = $mod->isa( 'CPAN::Distribution' )
	  ? $mod
	    : CPAN::Shell->expand( 'Distribution', $mod->cpan_file );

	$dist->install;

    }
    else
    {
	CPAN::Shell->install( $mod->id );
    }

    if ( $self->{force_install} || $::opt{force_install})
    {
        warn( "Forcing install\n" );
	system( "cd " . $mod->dir . " && make install " ) ==0 
          or die( "error in installation\n" );
    }

}


sub pathsubst
{
    my $self = shift;

    my $path = catfile( @_ );
    my @paths = split(':', $SysArch::Var{platforms} );

    my @substs = SysPathSubst::syspathsubst( undef, \@paths, [ $path ] );

    return wantarray ? @substs : $substs[0];
}

sub patch
{
    my ( $self ) = @_;

    my $mod = $self->{mod};

    (my $mod_id = $mod->id ) =~ s/::/-/g;

    my $patchfile = catfile( $::opt{patchdir},
			     $mod_id . '-' . $mod->cpan_version . '.patch' );
    if ( -e $patchfile )
    {
	$mod->get;
	my $dir = $mod->dir;
	::msg( "patching ", $mod->id, " with $patchfile\n" );

	if ( _requires_applypatch( $patchfile ) )
	{
	    $::opt{applypatch} ||= catfile( scalar $self->findexe( 'applypatch' ),
					    'applypatch');

	    my $cmd = "cd $dir; $::opt{applypatch} $patchfile";
	    warn("$cmd\n");
	    system( $cmd ) == 0
	      or die( "error installing patch for ", $mod->id, "\n" );

	}
	else
	{
	    $::opt{patch} ||= catfile( scalar $self->findexe( 'patch' ), 'patch');
	    system( "cd $dir; $::opt{patch} -p1 < $patchfile" ) == 0
	      or die( "error installing patch for ", $mod->id, "\n" );
	}

    }

}


sub findexe
{
    my ( $self, $exe, $path ) = @_;

    my $PATH ||= $ENV{PATH};

    my @paths = grep { -x catfile( $_, $exe ) && ! -d _ }
                          split(':', $PATH);

    die( "unable to find executable ($exe) in path $PATH\n" )
      unless @paths;

    return wantarray ? @paths : $paths[0];
}

sub findfile
{
    my ( $self, $file, @dirs ) = @_;

    my @paths = grep { -e catfile( $_, $file ) }
                   map { split(':', $_) }
		     @dirs;

    return wantarray ? @paths : $paths[0];
}


sub require
{
    my $self = shift;

    ::install( @_ );
    eval "require $_" foreach @_;
}

sub _requires_applypatch
{
    my ( $file ) = @_;

    open( my $fh, $file )
      or die( "unable to open $file\n" );

    my $requires_applypatch = grep { /ApplyPatch data/ } <$fh>;
    close $fh;

    return $requires_applypatch;
}

sub slurp
{
    my ( $self, $file ) = @_;

    my $fh = IO::File->new( $file )
      or die( "unable to open $file\n" );

    return wantarray ? $fh->getlines : join( '', $fh->getlines );
}


1;
